﻿/**
 * Progression 4
 * 
 * @author Copyright (C) 2007-2010 taka:nium.jp, All Rights Reserved.
 * @version 4.0.12
 * @see http://progression.jp/
 * 
 * Progression Libraries is dual licensed under the "Progression Library License" and "GPL".
 * http://progression.jp/license
 */
package jp.progression.casts.mx {
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.Stage;
	import flash.events.Event;
	import flash.utils.getDefinitionByName;
	import jp.nium.collections.IIdGroup;
	import jp.nium.core.debug.Logger;
	import jp.nium.core.L10N.L10NNiumMsg;
	import jp.nium.core.ns.nium_internal;
	import jp.nium.display.ExMovieClip;
	import jp.nium.events.ExEvent;
	import jp.nium.utils.ObjectUtil;
	import jp.progression.core.impls.ICastObject;
	import jp.progression.core.impls.IManageable;
	import jp.progression.core.ns.progression_internal;
	import jp.progression.core.proto.Configuration;
	import jp.progression.events.ManagerEvent;
	import jp.progression.executors.ExecutorObject;
	import jp.progression.Progression;
	import mx.core.Application;
	
	/**
	 * <span lang="ja">SWF ファイルの読み込みが完了し、stage 及び loaderInfo にアクセス可能になった場合に送出されます。</span>
	 * <span lang="en">Dispatch when the SWF file completes to read and the stage and the loaderInfo becomes able to access.</span>
	 * 
	 * @eventType jp.nium.events.ExEvent.EX_READY
	 */
	[Event( name="exReady", type="jp.nium.events.ExEvent" )]
	
	/**
	 * <span lang="ja">Progression インスタンスとの関連付けがアクティブになったときに送出されます。</span>
	 * <span lang="en"></span>
	 * 
	 * @eventType jp.progression.events.ManagerEvent.MANAGER_ACTIVATE
	 */
	[Event( name="managerActivate", type="jp.progression.events.ManagerEvent" )]
	
	/**
	 * <span lang="ja">Progression インスタンスとの関連付けが非アクティブになったときに送出されます。</span>
	 * <span lang="en"></span>
	 * 
	 * @eventType jp.progression.events.ManagerEvent.MANAGER_DEACTIVATE
	 */
	[Event( name="managerDeactivate", type="jp.progression.events.ManagerEvent" )]
	
	/**
	 * <span lang="ja">CastApplication クラスは、Application クラスの基本機能を拡張し、イベントフローとの連携機能を実装した jp.progression パッケージで使用される基本的な表示オブジェクトクラスです。</span>
	 * <span lang="en"></span>
	 * 
	 * @example <listing version="3.0">
	 * // CastApplication インスタンスを作成する
	 * var cast:CastApplication = new CastApplication();
	 * </listing>
	 */
	public class CastApplication extends Application implements ICastObject, IManageable {
		
		/**
		 * <span lang="ja">表示オブジェクトのステージです。</span>
		 * <span lang="en">The Stage of the display object.</span>
		 */
		public static function get stage():Stage { return _stage; }
		private static var _stage:Stage;
		
		
		
		
		
		/**
		 * <span lang="ja">自身の参照を取得します。</span>
		 * <span lang="en"></span>
		 */
		public function get self():CastApplication { return CastApplication( this ); }
		
		/**
		 * <span lang="ja">インスタンスの識別子を取得または設定します。</span>
		 * <span lang="en">Indicates the instance id of the IExDisplayObject.</span>
		 * 
		 * @see jp.nium.display#getInstanceById()
		 */
		override public function get id():String { return super.id; }
		override public function set id( value:String ):void {
			// 既存の登録を取得する
			var instance:IIdGroup = ExMovieClip.nium_internal::$collection.getInstanceById( value );
			
			// すでに存在していれば例外をスローする
			if ( instance ) { throw new Error( Logger.getLog( L10NNiumMsg.getInstance().ERROR_022 ).toString( value ) ); }
			
			// 登録する
			super.id = ExMovieClip.nium_internal::$collection.registerId( this, value ) ? value : null;
		}
		
		/**
		 * <span lang="ja">インスタンスのグループ名を取得または設定します。</span>
		 * <span lang="en">Indicates the instance group of the IExDisplayObject.</span>
		 * 
		 * @see jp.nium.display#getInstancesByGroup()
		 */
		public function get group():String { return _group; }
		public function set group( value:String ):void { _group = ExMovieClip.nium_internal::$collection.registerGroup( this, value ) ? value : null;  }
		private var _group:String;
		
		/**
		 * <span lang="ja">関連付けられている Progression インスタンスを取得します。</span>
		 * <span lang="en"></span>
		 * 
		 * @see jp.progression.events.ManagerEvent#MANAGER_ACTIVATE
		 * @see jp.progression.events.ManagerEvent#MANAGER_DEACTIVATE
		 */
		public function get manager():Progression { return _manager; }
		private var _manager:Progression;
		
		/**
		 * <span lang="ja">関連付けられている ExecutorObject インスタンスを取得します。</span>
		 * <span lang="en"></span>
		 * 
		 * @see #onCastAdded
		 * @see #onCastRemoved
		 * @see #addCommand()
		 * @see #insertCommand()
		 * @see #clearCommand()
		 */
		public function get executor():ExecutorObject { return _executor; }
		private var _executor:ExecutorObject;
		
		/**
		 * <span lang="ja">ドキュメントの準備が完了しているかどうかを取得します。</span>
		 * <span lang="en"></span>
		 */
		public function get isReady():Boolean { return _isReady; }
		private var _isReady:Boolean = false;
		
		/**
		 * ステージ上に設置されたかどうかを取得します。
		 */
		private var _isAddedToStage:Boolean = false;
		
		/**
		 * 
		 */
		private var _isCreationComplete:Boolean = false;
		
		/**
		 * ステージサイズが取得可能になったかどうかを取得します。
		 */
		private var _isStageSize:Boolean = false;
		
		/**
		 * 読み込み処理が完了したかどうかを取得します。
		 */
		private var _isComplete:Boolean = false;
		
		/**
		 * <span lang="ja">SWF ファイルの読み込みが完了し、stage 及び loaderInfo にアクセス可能になった場合に呼び出されるイベントハンドラメソッドを取得または設定します。</span>
		 * <span lang="en"></span>
		 * 
		 * @see jp.nium.events.ExEvent#EX_READY
		 */
		public function get onReady():Function { return _onReady; }
		public function set onReady( value:Function ):void { _onReady = value; }
		private var _onReady:Function;
		
		/**
		 * <span lang="ja">SWF ファイルの読み込みが完了し、stage 及び loaderInfo にアクセス可能になった場合に呼び出されるオーバーライド・イベントハンドラメソッドです。</span>
		 * <span lang="en"></span>
		 */
		protected function atReady():void {}
		
		/**
		 * @private
		 */
		public function get onCastAdded():Function { throw new Error( Logger.getLog( L10NNiumMsg.getInstance().ERROR_001 ).toString( "onCastAdded" ) ); }
		public function set onCastAdded( value:Function ):void { throw new Error( Logger.getLog( L10NNiumMsg.getInstance().ERROR_001 ).toString( "onCastAdded" ) ); }
		
		/**
		 * @private
		 */
		protected final function atCastAdded():void {}
		
		/**
		 * @private
		 */
		public function get onCastRemoved():Function { throw new Error( Logger.getLog( L10NNiumMsg.getInstance().ERROR_001 ).toString( "onCastRemoved" ) ); }
		public function set onCastRemoved( value:Function ):void { throw new Error( Logger.getLog( L10NNiumMsg.getInstance().ERROR_001 ).toString( "onCastRemoved" ) ); }
		
		/**
		 * @private
		 */
		protected final function atCastRemoved():void {}		
		
		
		
		
		/**
		 * <span lang="ja">新しい CastApplication インスタンスを作成します。</span>
		 * <span lang="en">Creates a new CastApplication object.</span>
		 */
		public function CastApplication() {
			// クラスをコンパイルに含める
			nium_internal;
			progression_internal;
			
			// イベントリスナーを登録する
			super.addEventListener( Event.ADDED_TO_STAGE, _addedToStage );
			super.addEventListener( "creationComplete", _creationComplete );
			super.addEventListener( ExEvent.EX_READY, _ready, false, int.MAX_VALUE );
		}
		
		
		
		
		
		/**
		 * <span lang="ja">マネージャーオブジェクトとの関連付けを更新します。</span>
		 * <span lang="en"></span>
		 * 
		 * @return
		 * <span lang="ja">関連付けが成功したら true を、それ以外は false を返します。</span>
		 * <span lang="en"></span>
		 * 
		 * @see #manager
		 * @see jp.progression.Progression
		 */
		public function updateManager():Boolean {
			return Boolean( !!_manager );
		}
		
		/**
		 * <span lang="ja">特定のイベントが送出された際に、自動実行させたい Command インスタンスをリストの最後尾に追加します。
		 * 追加された Command インスタンスは、イベントが送出される直前に自動的に初期化されます。
		 * このメソッドを使用するためには executor プロパティに CommandExecutor が実装されている必要があります。</span>
		 * <span lang="en"></span>
		 * 
		 * @param commands
		 * <span lang="ja">登録したいコマンドを含む配列です。</span>
		 * <span lang="en"></span>
		 * 
		 * @see #executor
		 * @see #onCastAdded
		 * @see #onCastRemoved
		 * @see #insertCommand()
		 * @see #clearCommand()
		 */
		public function addCommand( ... commands:Array ):void {
			ExecutorObject.progression_internal::$addCommand( _executor, commands );
		}
		
		/**
		 * <span lang="ja">特定のイベントが送出された際に、自動実行させたい Command インスタンスをすでにリストに登録され、実行中の Command インスタンスの次の位置に追加します。
		 * 追加された Command インスタンスは、イベントが送出される直前に自動的に初期化されます。
		 * このメソッドを使用するためには executor プロパティに CommandExecutor が実装されている必要があります。</span>
		 * <span lang="en"></span>
		 * 
		 * @param commands
		 * <span lang="ja">登録したいコマンドを含む配列です。</span>
		 * <span lang="en"></span>
		 * 
		 * @see #executor
		 * @see #onCastAdded
		 * @see #onCastRemoved
		 * @see #addCommand()
		 * @see #clearCommand()
		 */
		public function insertCommand( ... commands:Array ):void {
			ExecutorObject.progression_internal::$insertCommand( _executor, commands );
		}
		
		/**
		 * <span lang="ja">登録されている Command インスタンスを削除します。
		 * このメソッドを使用するためには executor プロパティに CommandExecutor が実装されている必要があります。</span>
		 * <span lang="en"></span>
		 * 
		 * @param completely
		 * <span lang="en">true が設定されている場合は登録されている全てのコマンド登録を解除し、false の場合には現在処理中のコマンド以降の登録を解除します。</span>
		 * <span lang="en"></span>
		 * 
		 * @see #executor
		 * @see #onCastAdded
		 * @see #onCastRemoved
		 * @see #addCommand()
		 * @see #insertCommand()
		 */
		public function clearCommand( completely:Boolean = false ):void {
			ExecutorObject.progression_internal::$clearCommand( _executor, completely );
		}
		
		/**
		 * @private
		 */
		public function dispose():void {
			throw new Error( Logger.getLog( L10NNiumMsg.getInstance().ERROR_000 ).toString( "dispose" ) );
		}
		
		/**
		 * 
		 */
		private function _checkReady():Boolean {
			if ( _isReady ) { return true; }
			if ( !_isAddedToStage || !_isStageSize || !_isComplete || !_isCreationComplete ) { return false; }
			
			// 状態を変更する
			_isReady = true;
			
			// イベントを送出する
			_stage.dispatchEvent( new ExEvent( ExEvent.EX_RESIZE_START ) );
			_stage.dispatchEvent( new ExEvent( ExEvent.EX_RESIZE_PROGRESS ) );
			_stage.dispatchEvent( new ExEvent( ExEvent.EX_RESIZE_COMPLETE ) );
			super.dispatchEvent( new ExEvent( ExEvent.EX_READY ) );
			
			return true;
		}
		
		/**
		 * <span lang="ja">インスタンスに対して、複数のプロパティを一括設定します。</span>
		 * <span lang="en">Setup the several instance properties.</span>
		 * 
		 * @param parameters
		 * <span lang="ja">設定したいプロパティを含んだオブジェクトです。</span>
		 * <span lang="en">The object that contains the property to setup.</span>
		 * @return
		 * <span lang="ja">自身の参照です。</span>
		 * <span lang="en"></span>
		 */
		public function setProperties( parameters:Object ):DisplayObject {
			ObjectUtil.setProperties( this, parameters );
			return this;
		}
		
		/**
		 * <span lang="ja">指定されたオブジェクトのストリング表現を返します。</span>
		 * <span lang="en">Returns the string representation of the specified object.</span>
		 * 
		 * @return
		 * <span lang="ja">オブジェクトのストリング表現です。</span>
		 * <span lang="en">A string representation of the object.</span>
		 */
		override public function toString():String {
			return ObjectUtil.formatToString( this, super.className, super.id ? "id" : null );
		}
				
		
		
		
		
		/**
		 * 表示オブジェクトが、直接、または表示オブジェクトを含むサブツリーの追加により、ステージ上の表示リストに追加されたときに送出されます。
		 */
		private function _addedToStage( e:Event ):void {
			// イベントリスナーを解除する
			super.removeEventListener( Event.ADDED_TO_STAGE, _addedToStage );
			
			// stage の参照を保存する
			_stage = super.stage;
			
			// ルートクラスを取得する
			var cls:Class = Object( this ).constructor as Class;
			var config:Configuration = Object( cls ).config as Configuration;
			var managerId:String = Object( cls ).managerId as String;
			var rootClass:Class = Object( cls ).rootClass as Class;
			
			// 初期化する
			Progression.initialize( config );
			
			// Application を取得する
			var application:Object;
			try { application ||= getDefinitionByName( "mx.core.FlexGlobals" ).topLevelApplication; } catch ( e:Error ) {}
			try { application ||= getDefinitionByName( "mx.core.Application" ).application; } catch ( e:Error ) {}
			
			// Progression インスタンスを作成する
			_manager = new Progression( managerId, application as DisplayObjectContainer || this, rootClass );
			
			// イベントを送出する
			super.dispatchEvent( new ManagerEvent( ManagerEvent.MANAGER_ACTIVATE, false, false, _manager ) );
			
			// ExecutorObject を作成する
			var classRef:Class = Progression.config.executor;
			_executor = new classRef( this );
			
			// コンテクストメニューを設定する
			this["_contextMenuBuilder"] = Progression.config.contextMenuBuilder;
			
			// イベントリスナーを登録する
			super.addEventListener( Event.ENTER_FRAME, _enterFrame );
			
			// 既に読み込みが完了していれば
			if ( super.loaderInfo.bytesTotal > 0 && super.loaderInfo.bytesLoaded >= super.loaderInfo.bytesTotal ) {
				_complete( null );
			}
			else {
				super.loaderInfo.addEventListener( Event.COMPLETE, _complete );
			}
			
			// 状態を変更する
			_isAddedToStage = true;
			
			// 準備が完了したかどうか確認する
			_checkReady();
		}
		
		/**
		 * 
		 */
		private function _creationComplete( e:Event ):void {
			// イベントリスナーを解除する
			super.removeEventListener( "creationComplete", _creationComplete );
			
			// 状態を変更する
			_isCreationComplete = true;
			
			// 準備が完了したかどうか確認する
			_checkReady();
		}
		
		/**
		 * データが正常にロードされたときに送出されます。
		 */
		private function _complete( e:Event ):void {
			// イベントリスナーを解除する
			super.loaderInfo.removeEventListener( Event.COMPLETE, _complete );
			
			// 状態を変更する
			_isComplete = true;
			
			// 準備が完了したかどうか確認する
			_checkReady();
		}
		
		/**
		 * 
		 */
		private function _enterFrame( e:Event ):void {
			if ( stage.stageWidth <= 0 || stage.stageHeight <= 0 ) { return; }
			
			// イベントリスナーを解除する
			super.removeEventListener( Event.ENTER_FRAME, _enterFrame );
			
			// 状態を変更する
			_isStageSize = true;
			
			// 準備が完了したかどうか確認する
			_checkReady();
		}
		
		/**
		 * SWF ファイルの読み込みが完了し、stage 及び loaderInfo にアクセス可能になった場合に送出されます。
		 */
		private function _ready( e:ExEvent ):void {
			// イベントリスナーを解除する
			super.removeEventListener( ExEvent.EX_READY, _ready );
			
			// イベントハンドラメソッドを実行する
			( _onReady as Function || atReady as Function ).apply( this );
		}
		
		
		
		
		
		/**
		 * 外部 ActionScript ファイルを取り込む
		 */
		include "../../core/includes/CastObject_contextMenu.inc"
	}
}
