﻿/**
 * Progression 4
 * 
 * @author Copyright (C) 2007-2010 taka:nium.jp, All Rights Reserved.
 * @version 4.0.12
 * @see http://progression.jp/
 * 
 * Progression Libraries is dual licensed under the "Progression Library License" and "GPL".
 * http://progression.jp/license
 */
package jp.progression.core.managers {
	import flash.events.Event;
	import flash.utils.getDefinitionByName;
	import jp.nium.core.debug.Logger;
	import jp.progression.config.MXMLWebConfig;
	import jp.progression.core.L10N.L10NProgressionMsg;
	import jp.progression.core.PackageInfo;
	import jp.progression.Progression;
	import org.libspark.ui.SWFSize;
	import org.libspark.ui.SWFWheel;
	
	/**
	 * @private
	 */
	public class MXMLWebInitializer implements IInitializer {
		
		/**
		 * <span lang="ja">関連付けられている Progression インスタンスを取得します。</span>
		 * <span lang="en"></span>
		 */
		public function get target():Progression { return _target; }
		private var _target:Progression;
		
		
		
		
		
		/**
		 * <span lang="ja">新しい MXMLWebInitializer インスタンスを作成します。</span>
		 * <span lang="en">Creates a new MXMLWebInitializer object.</span>
		 * 
		 * @param target
		 * <span lang="ja">関連付けたい Progression インスタンスです。</span>
		 * <span lang="en"></span>
		 */
		public function MXMLWebInitializer( target:Progression ) {
			// 引数を設定する
			_target = target;
			
			// 現在の環境設定を取得します。
			var config:MXMLWebConfig = Progression.config as MXMLWebConfig;
			
			// 環境設定が MXMLConfig であり、SWFWheel を使用するのであれば
			if ( config && config.useSWFWheel ) {
				// SWFWheel を初期化する
				SWFWheel.initialize( _target.stage );
				
				if ( PackageInfo.hasDebugger ) {
					// 情報を表示する
					Logger.info( Logger.getLog( L10NProgressionMsg.getInstance().INFO_000 ).toString( "SWFWheel" ) );
				}
			}
			
			// 環境設定が WebConfig ではない、または SWFSize を使用しないのであれば
			if ( !config || !config.useSWFSize ) { return; }
			
			// 既に読み込まれていれば
			if ( _target.stage.loaderInfo.bytesTotal > 0 && _target.stage.loaderInfo.bytesLoaded >= _target.stage.loaderInfo.bytesTotal ) {
				_complete( null );
			}
			else {
				_target.stage.loaderInfo.addEventListener( Event.COMPLETE, _complete );
			}
		}
		
		
		
		
		
		/**
		 * <span lang="ja">破棄します。</span>
		 * <span lang="en"></span>
		 */
		public function dispose():void {
			// 破棄する
			_target = null;
		}
		
		
		
		
		
		/**
		 * データが正常にロードされたときに送出されます。
		 */
		private function _complete( e:Event ):void {
			// イベントリスナーを解除する
			_target.stage.loaderInfo.removeEventListener( Event.COMPLETE, _complete );
			
			// Application を取得する
			var application:Object;
			try { application ||= getDefinitionByName( "mx.core.FlexGlobals" ).topLevelApplication; } catch ( e:Error ) {}
			try { application ||= getDefinitionByName( "mx.core.Application" ).application; } catch ( e:Error ) {}
			
			// SWFSize を初期化する
			SWFSize.initialize( application.loaderInfo.width, application.loaderInfo.height );
			
			if ( PackageInfo.hasDebugger ) {
				// 情報を表示する
				Logger.info( Logger.getLog( L10NProgressionMsg.getInstance().INFO_000 ).toString( "SWFSize" ) );
			}
		}
	}
}
