package scenes {
	import jp.progression.casts.*;
	import jp.progression.casts.mx.*;
	import jp.progression.commands.*;
	import jp.progression.commands.display.*;
	import jp.progression.commands.lists.*;
	import jp.progression.commands.net.*;
	import jp.progression.commands.tweens.*;
	import jp.progression.data.*;
	import jp.progression.events.*;
	import jp.progression.executors.*;
	import jp.progression.scenes.*;
	
	import mx.controls.Label;
	import mx.core.UIComponent;
	
	/**
	 * ...
	 * @author ...
	 */
	public class IndexScene extends SceneObject {
		private var _topLabel:UIComponent;
		
		/**
		 * 新しい IndexScene インスタンスを作成します。
		 */
		public function IndexScene() {
			// シーンタイトルを設定します。
			title = "MyProject";
			
			addScene(new DataGridScene());
			addScene(new RichTextScene());
		}
		
		/**
		 * シーン移動時に目的地がシーンオブジェクト自身もしくは子階層だった場合に、階層が変更された直後に送出されます。
		 * このイベント処理の実行中には、ExecutorObject を使用した非同期処理が行えます。
		 */
		override protected function atSceneLoad():void {
			// インスタンスを作成
			var label:Label = new Label();
			label.text = "Flex と Progression の連携デモ";
			label.scaleX = label.scaleY = 4;
			container.addChild(label);
			
			addCommand(
				new DoTweener(label, {scaleX:1, scaleY:1, time:2})
			);
		}
		
		/**
		 * シーンオブジェクト自身が目的地だった場合に、到達した瞬間に送出されます。
		 * このイベント処理の実行中には、ExecutorObject を使用した非同期処理が行えます。
		 */
		override protected function atSceneInit():void {
			_topLabel = new UIComponent();
			_topLabel.graphics.beginFill(0xFF0000);
			_topLabel.graphics.drawCircle(0,0,100);
			container.addChild(_topLabel);
			
			_topLabel.scaleX = 0;
			_topLabel.scaleY = 0;
			
			addCommand(
				new DoTweener(_topLabel, {
					scaleX:1, 
					scaleY:1, 
					y:200,
					time:1.5, 
					transition:"easeOutElastic"
					})
			);
		}
		
		/**
		 * シーンオブジェクト自身が出発地だった場合に、移動を開始した瞬間に送出されます。
		 * このイベント処理の実行中には、ExecutorObject を使用した非同期処理が行えます。
		 */
		override protected function atSceneGoto():void {
			addCommand(
				new DoTweener(_topLabel, {
					scaleX:0, 
					scaleY:0, 
					time:0.7, 
					transition:"easeInBack"
				}),
				new Func(function():void{
					container.removeChild(_topLabel);
					_topLabel = null;
				})
			);
			
		}
		
		/**
		 * シーン移動時に目的地がシーンオブジェクト自身もしくは親階層だった場合に、階層が変更される直前に送出されます。
		 * このイベント処理の実行中には、ExecutorObject を使用した非同期処理が行えます。
		 */
		override protected function atSceneUnload():void {
			addCommand(
			);
		}
	}
}
